<?php
namespace Controllers;

use Services\OAuth2 as Service;
use Zeedhi\Framework\DTO;
use Util\InstanceProvider;

class OAuth2 {
    const SECRET = 'wso2';

    /** @var Service */
    private $service;

    /**
     * Constructor.
     * @param Service $service The OAuth2 Service.
     */
    public function __construct(Service $service) {
        header('Content-Type: application/json');
        $this->service = $service;
    }

    /**
     * Authenticates the requester user.
     * @param DTO\Request $request   The Request Object.
     * @param DTO\Response $response The Response Object.
     */
    public function authenticate(DTO\Request $request, DTO\Response $response) {
        $grantType = $request->getParameter('grant_type');
        $requesterUrl = $_SERVER['HTTP_REFERER'];
        $productId = $_SERVER['PHP_AUTH_USER'];

        if($grantType === 'password') {
            $user = $request->getParameter('username');
            $password = $request->getParameter('password');
        } elseif($grantType === 'refresh_token') {
            $refreshToken = $request->getParameter('refresh_token');
            $session = InstanceProvider::getSession();
            $loginData = json_decode(
                $session->get(
                    $refreshToken
                ),
                true
            );
            $user = $loginData['username'];
            $password = $loginData['password'];
            $session->remove($refreshToken);
        }

        $userData = $this->service->login($user, $password, $productId, $requesterUrl, 0);

        $header = $this->getHeader();
        $payload = $this->getPayload($userData, $requesterUrl);

        $jwt = $this->getJWT($header, $payload);

        die($this->buildResponse($jwt, $userData['TOKEN'], $userData['REFRESH_TOKEN'], $userData['EXPIRES_IN']));
    }

    /**
     * Builds the JSON Response.
     *
     * @param string $jwt           The jwt token.
     * @param string $accessToken   The access token.
     * @param string $refreshToken  The refresh token.
     * @param int    $expiresIn     The expiration time.
     *
     * @return string The JSON Response.
     */
    private function buildResponse($jwt, $accessToken = "", $refreshToken = "", $expiresIn = 3600) {
        $response = [
            'jwt_token' => $jwt,
            'access_token' => $accessToken,
            'refresh_token' => $refreshToken,
            'token_type' => 'Bearer',
            'expires_in' => $expiresIn
        ];
        return json_encode($response);
    }

    /**
     * Retrieves a JWT token.
     *
     * @param array $header The information about token type and algorithm.
     * @param array $header The information about the user.
     *
     * @return string The JWT.
     */
    private function getJWT($header, $payload){
        $base64UrlHeader = $this->getBase64UrlString($header);
        $base64UrlPayload = $this->getBase64UrlString($payload);
        $base64UrlSignature = $this->getBase64Signature(
            $base64UrlHeader,
            $base64UrlPayload,
            self::SECRET
        );

        return $this->buildJWT(
            $base64UrlHeader,
            $base64UrlPayload,
            $base64UrlSignature
        );
    }

    /**
     * Generates a base 64 url string from an array.
     *
     * @param array $data The array to parse.
     *
     * @return string
     */
    private function getBase64UrlString($data){
        return str_replace(
            ['+', '/', '='],
            ['-', '_', ''],
            base64_encode(json_encode($data))
        );
    }

    /**
     * Generates a base 64 url signature using HS256 algorithm.
     *
     * @param string $header    The base 64 header.
     * @param string $payload   The base 64 payload.
     * @param string $secret    The secret to encrypt the signature over.
     *
     * @return string
     */
    private function getBase64Signature($header, $payload, $secret) {
        /** @var string $signature The result of HS256 algorithm over the $secret string. */
        $signature = hash_hmac('sha256', $header . "." . $payload, $secret, true);

        return str_replace(
            ['+', '/', '='],
            ['-', '_', ''],
            base64_encode($signature)
        );
    }

    /**
     * Builds a JWT token from given strings.
     *
     * @param string $base64UrlHeader    The b64 encoded header.
     * @param string $base64UrlPayload   The b64 encoded payload.
     * @param string $base64UrlSignature The b64 encoded signature.
     *
     * @return string
     */
    private function buildJWT($base64UrlHeader, $base64UrlPayload, $base64UrlSignature){
        return $base64UrlHeader . "." . $base64UrlPayload . "." . $base64UrlSignature;
    }

    /**
     * Returns the header.
     *
     * @return array The header.
     */
    private function getHeader(){
        return [
            'typ' => 'JWT',
            'alg' => 'HS256'
        ];
    }

    /**
     * Returns the payload.
     *
     * @param array  $userData     The user's session data.
     * @param string $requesterUrl The requester Url.
     *
     * @return array The payload.
     */
    private function getPayload($userData, $requesterUrl){
        return [
            'sub' => $userData['USER_ID'],
            'name' => $userData['USER_NAME'],
            'iss' => $requesterUrl,
            'iat' => time(),
            'exp' => $userData['EXPIRES_IN']
        ];
    }
}